##################################################################
####  
####  For exact local copy number recognition with GAP one needs
####  1. Segmented and smoothed Log R ratio profile
####  2. Segmented and smoothed B allele frequency profile
#### 
####  Columns of Input file: segmented and processed LRR and BAF profiles
####  "_segmented" data file is space delimited text file with following columns
####   Ind    SNP index in corresponding platform, specifying start of segment
####   Ind_K  SNP index in corresponding platform, specifying end of segment
####   Len    the length of the segment 
####   Chr    Chromosome number (if integer, p arm, if .5 q arm)
####   M_LRR  Median Log R ratio in the segment 
####   M_BAF  Mode of major BAF distribution
####   LOH    1, if germline homozygous, 0 otherwize
####   LenHete Number of informative SNPs in the segment (mBAF<0.97)
#### 
####
#########################  
##################################################################

###############################################
### File paths, other parameters and functions

source("C:/Documents and Settings/Tania/Bureau/homepage/SNP_GAP/Source/script_parameters_GAP.txt")
source("C:/Documents and Settings/Tania/Bureau/homepage/SNP_GAP/Source/script_functions_GAP.txt")


###### Reading sampleNameFile, the file with sample names

sampleNames<-read.csv(paste(inputPath,sampleNameFile,sep=""),header=T,fill=T)
nSample<-dim(sampleNames)[[1]]


#################################################################
### Cycle for processing samples one by one

ii<-nSample

for(ii in 4:nSample){


##################################################################
####
#### Step 1.  Reading
####
##################################################################


print(ii)

dataFName<-paste(inputPath,segmFilePrefix,as.character(sampleNames[ii,1]),".txt",sep="")

seg<-read.table(dataFName,sep = " ", quote = "\"'",dec = ".")

seg<-data.matrix(seg)

c_is<-1
c_if<-2
c_len<-3
c_chr<-4
c_lrr<-5
c_baf<-6
c_loh<-7
c_lenH<-8


#################################################
##### 1.1 23,24,25 Chromosomes segments are better to filter before fitting

tmp<-seg

tt<-which(tmp[,c_chr]>22.8);if(length(tt)>0){tmp<-tmp[-tt,]}

##################################################################
####
#### Step 2.  Copy number recognition
####
##################################################################

result<-CopyNumber_and_Genotype(tmp[,c_lrr],tmp[,c_baf],tmp[,c_len],showGAPTemplate=T)

###  matrix of segments with recognized copy numbers and genotypes
tmp<-cbind(tmp,result$tmp[,5:19])

### matrix of clusters on the LRR x BAF plane
NETM<-result$NETM

### matrix of coordinates of claste centers
Centr_CL<-result$Centr_CL

### matrix of coordinates of subclones
SB_CL<-result$SB_CL

### coefficients of the pattern shrinkage
p_BAF<-result$coeff[1]
q_LRR<-result$coeff[2]

rm(result)


##### 23,24,25 chromosomes back, if necessary

#tt<-which(seg[,c_chr]>22.8);
#if(length(tt)>0){
#for(j in 1:length(tt)){
#tmp<-rbind(tmp,tmp[1,]*0)
#tmp[dim(tmp)[[1]],1:dim(seg)[[2]]]<-seg[tt[j],]
#tmp[dim(tmp)[[1]],(dim(seg)[[2]]+1):dim(tmp)[[2]]]<-NA
#}
#}



##################################################################
####
#### Step 3.  Saving processed file and GAP plot
####
##################################################################

####  Output columns
#
#  n_LRR           subsidiary column for LRR 
#  n_BAF          subsidiary column for BAF
#  CN             final copy number status/ for subclones estimation of CN
#  BA             final number of B alleles, 0 for germline homozygosity/ for subclones estimation of CN/BA
#  Confidence     confidence in final attributions (1-confident, 4-questionable)
#  Subclone       for subclones estimation of BA
#  CN1,BA1,Rank1  CN and BA status of the closest cluster, Rank=distance to cluster
#  CN2,BA2,Rank2  CN and BA status of the second close cluster, Rank=distance to cluster
#  CN3,BA3,Rank3  CN and BA status of the third close cluster, Rank=distance to cluster
#
###   



IDF1<-paste(outputPath,processedFilePrefix,as.character(sampleNames[ii,1]),".txt",sep="")
write.table(tmp, file = IDF1, quote=F)


saveGAPplot<-T

if(saveGAPplot){


FigName<-paste(plotOutputPath,plotPrefix,as.character(sampleNames[ii,1]),".jpg",sep="")

#postscript(FigName,width=4.6,height=5.8)


jpeg(file=FigName,height=1000,width=1200,pointsize=24,quality=100)


plot(tmp[,c_baf],tmp[,c_lrr],type="n",main=as.character(sampleNames[ii,1]),xlim=c(0,1),ylim=c(-1,1),xlab="B Allele Frequency",ylab="Log R ratio")

CEX<-4*tmp[,c_len]/quantile(tmp[,c_len],probs=0.98)+0.4

while(CEX[which.max(CEX)]>4){CEX[which.max(CEX)]<-4}
points(tmp[,c_baf],tmp[,c_lrr],cex=CEX,col="grey60")
points(1-tmp[,c_baf],tmp[,c_lrr],cex=CEX,col="black")

abline(v=0.5,col="black")
abline(h=0,col="black")

RB<-rainbow(40)

for(p in 1:38){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){
		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])
	}
}

for(j in 1:dim(Centr_CL)[[2]]){

	if(Centr_CL[1,j]==Centr_CL[2,j]*2){
		text(0.54,Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))

	}else{
		text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
		}
}


text(0.25,-0.8,paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=0.8)
text(0.25,-0.9,paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=0.8)

dev.off()    

}# if plot


}# ii



##################################################################
####  Step 4. Improving results
####
####  Obtained results could help in improving segmentation and 
####  optimising breakpoints by
#### 1. Correcting connection defects
#### 2. Adjusting non-confident attributions to confident surroundings 
####
#### Procedure is really effective for low quality and highly contaminated samples
#### In order to recalculate means one needs initial LRR and mBAF SNP profiles
#### R_sampleName file
##################################################################

Step4 is still under testing and will be available in some time